JSS_CONFIG = {
    'end_point': 'http://storage.jd.com',
    'access_key': '',
    'secret_key': '',
    'JSS_HEADER_PREFIX': 'x-jss-',
    'COPY_SOURCE': 'x-jss-copy-source',
    'X_JSS_REQUETS_ID': 'x-jss-request-id',
    'JSON_TYPE': 'application/json, charset=UTF-8',
    'X_JSS_OBJECT_EXPIRES': 'x-jss-object-expires',
    'X_JSS_COPY_SOURCE': 'x-jss-copy-source',
    'X_JSS_METADATA_DIRECTIVE': 'x-jss-metadata-directive',
    'X_JSS_COPY_SOURCE_IF_MATCH': 'x-jss-copy-source-if-match',
    'X_JSS_COPY_SOURCE_IF_NONE_MATCH': 'x-jss-copy-source-if-none-match',
    'X_JSS_COPY_SOURCE_IF_UNMODIFIED_SINCE': 'x-jss-copy-source-if-unmodified-since',
    'X_JSS_COPY_SOURCE_IF_MODIFIED_SINCE': 'x-jss-copy-source-if-modified-since',
    'X_JSS_COPY_RANGE': 'x-jss-copy-range'
}
